@@ -179,7 +179,7 @@ bitunshuffle1_altivec(void* _src, void* dest, const size_t size, const size_t el
 
 
 /* Transpose bytes within elements for 16 bit elements. */
-int64_t bshuf_trans_byte_elem_16(void* in, void* out, const size_t size) {
+int64_t bshuf_trans_byte_elem_16(const void* in, void* out, const size_t size) {
   static const uint8_t bytesoftype = 2;
   __vector uint8_t xmm0[2];
 
@@ -199,7 +199,7 @@ int64_t bshuf_trans_byte_elem_16(void* in, void* out, const size_t size) {
 
 
 /* Transpose bytes within elements for 32 bit elements. */
-int64_t bshuf_trans_byte_elem_32(void* in, void* out, const size_t size) {
+int64_t bshuf_trans_byte_elem_32(const void* in, void* out, const size_t size) {
   static const uint8_t bytesoftype = 4;
   __vector uint8_t xmm0[4];
 
@@ -219,7 +219,7 @@ int64_t bshuf_trans_byte_elem_32(void* in, void* out, const size_t size) {
 
 
 /* Transpose bytes within elements for 64 bit elements. */
-int64_t bshuf_trans_byte_elem_64(void* in, void* out, const size_t size) {
+int64_t bshuf_trans_byte_elem_64(const void* in, void* out, const size_t size) {
   static const uint8_t bytesoftype = 8;
   __vector uint8_t xmm0[8];
 
@@ -239,7 +239,7 @@ int64_t bshuf_trans_byte_elem_64(void* in, void* out, const size_t size) {
 
 
 /* Transpose bytes within elements for 128 bit elements. */
-int64_t bshuf_trans_byte_elem_128(void* in, void* out, const size_t size) {
+int64_t bshuf_trans_byte_elem_128(const void* in, void* out, const size_t size) {
   static const uint8_t bytesoftype = 16;
   __vector uint8_t xmm0[16];
 
@@ -258,20 +258,8 @@ int64_t bshuf_trans_byte_elem_128(void* in, void* out, const size_t size) {
 }
 
 
-/* Memory copy with bshuf call signature. */
-int64_t bshuf_copy(void* in, void* out, const size_t size,
-                   const size_t elem_size) {
-
-  char* in_b = (char*)in;
-  char* out_b = (char*)out;
-
-  memcpy(out_b, in_b, size * elem_size);
-  return size * elem_size;
-}
-
-
 /* Transpose bytes within elements using best SSE algorithm available. */
-int64_t bshuf_trans_byte_elem_altivec(void* in, void* out, const size_t size,
+int64_t bshuf_trans_byte_elem_altivec(const void* in, void* out, const size_t size,
                                       const size_t elem_size, void* tmp_buf) {
 
   int64_t count;
@@ -338,7 +326,7 @@ int64_t bshuf_trans_byte_elem_altivec(void* in, void* out, const size_t size,
 
 
 /* Transpose bits within bytes. */
-int64_t bshuf_trans_bit_byte_altivec(void* in, void* out, const size_t size,
+int64_t bshuf_trans_bit_byte_altivec(const void* in, void* out, const size_t size,
                                      const size_t elem_size) {
 
   const uint8_t* in_b = (const uint8_t*)in;
@@ -372,25 +360,31 @@ int64_t bshuf_trans_bit_byte_altivec(void* in, void* out, const size_t size,
 
 
 /* Transpose bits within elements. */
-int64_t bshuf_trans_bit_elem_altivec(void* in, void* out, const size_t size,
-                                     const size_t elem_size, void* tmp_buf) {
+int64_t bshuf_trans_bit_elem_altivec(const void* in, void* out, const size_t size,
+                                     const size_t elem_size) {
 
   int64_t count;
 
   CHECK_MULT_EIGHT(size);
 
+  void* tmp_buf = malloc(size * elem_size);
+  if (tmp_buf == NULL) return -1;
+
   count = bshuf_trans_byte_elem_altivec(in, out, size, elem_size, tmp_buf);
   CHECK_ERR(count);
   // bshuf_trans_bit_byte_altivec / bitshuffle1_altivec
   count = bshuf_trans_bit_byte_altivec(out, tmp_buf, size, elem_size);
   CHECK_ERR(count);
   count = bshuf_trans_bitrow_eight(tmp_buf, out, size, elem_size);
+
+  free(tmp_buf);
+
   return count;
 }
 
 /* For data organized into a row for each bit (8 * elem_size rows), transpose
  * the bytes. */
-int64_t bshuf_trans_byte_bitrow_altivec(void* in, void* out, const size_t size,
+int64_t bshuf_trans_byte_bitrow_altivec(const void* in, void* out, const size_t size,
                                         const size_t elem_size) {
   static const __vector uint8_t epi8_low = (const __vector uint8_t) {
     0x00, 0x10, 0x01, 0x11, 0x02, 0x12, 0x03, 0x13,
@@ -541,7 +535,7 @@ int64_t bshuf_trans_byte_bitrow_altivec(void* in, void* out, const size_t size,
 
 
 /* Shuffle bits within the bytes of eight element blocks. */
-int64_t bshuf_shuffle_bit_eightelem_altivec(void* in, void* out, const size_t size,
+int64_t bshuf_shuffle_bit_eightelem_altivec(const void* in, void* out, const size_t size,
                                             const size_t elem_size) {
   /*  With a bit of care, this could be written such that such that it is */
   /*  in_buf = out_buf safe. */
@@ -579,17 +573,21 @@ int64_t bshuf_shuffle_bit_eightelem_altivec(void* in, void* out, const size_t si
 
 
 /* Untranspose bits within elements. */
-int64_t bshuf_untrans_bit_elem_altivec(void* in, void* out, const size_t size,
-                                       const size_t elem_size, void* tmp_buf) {
+int64_t bshuf_untrans_bit_elem_altivec(const void* in, void* out, const size_t size,
+                                       const size_t elem_size) {
 
   int64_t count;
 
   CHECK_MULT_EIGHT(size);
 
+  void* tmp_buf = malloc(size * elem_size);
+  if (tmp_buf == NULL) return -1;
+
   count = bshuf_trans_byte_bitrow_altivec(in, tmp_buf, size, elem_size);
   CHECK_ERR(count);
   count = bshuf_shuffle_bit_eightelem_altivec(tmp_buf, out, size, elem_size);
 
+  free(tmp_buf);
   return count;
 }
 
