@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2009-2018  Brazil
-  Copyright (C) 2019-2024  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2019-2025  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -187,7 +188,6 @@ grn_init_external_libraries(void)
   blosc2_init();
 #endif
   grn_distance_init_external_libraries();
-  grn::language_model::init_external_libraries();
 }
 
 static void
@@ -259,8 +259,8 @@ grn_loader_init(grn_loader *loader)
   loader->error.line = 0;
   loader->error.file = NULL;
   loader->error.func = NULL;
-  loader->output_ids = GRN_FALSE;
-  loader->output_errors = GRN_FALSE;
+  loader->output_ids = false;
+  loader->output_errors = false;
 }
 
 void
@@ -438,7 +438,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
     ctx->impl->match_escalation_threshold =
       grn_get_default_match_escalation_threshold();
   }
-  ctx->impl->force_match_escalation = GRN_FALSE;
+  ctx->impl->force_match_escalation = false;
 
   ctx->impl->finalizer = NULL;
 
@@ -453,7 +453,6 @@ grn_ctx_impl_init(grn_ctx *ctx)
 #endif
   ctx->impl->output.arrow_stream_writer = NULL;
   grn_timeval_now(ctx, &ctx->impl->tv);
-  ctx->impl->edge = NULL;
 #ifdef GRN_WITH_APACHE_ARROW
   ctx->impl->arrow_stream_loader = NULL;
 #endif
@@ -1405,18 +1404,18 @@ grn_ctx_set_match_escalation_threshold(grn_ctx *ctx, int64_t threshold)
   return GRN_SUCCESS;
 }
 
-grn_bool
+bool
 grn_ctx_get_force_match_escalation(grn_ctx *ctx)
 {
   if (ctx->impl) {
     return ctx->impl->force_match_escalation;
   } else {
-    return GRN_FALSE;
+    return false;
   }
 }
 
 grn_rc
-grn_ctx_set_force_match_escalation(grn_ctx *ctx, grn_bool force)
+grn_ctx_set_force_match_escalation(grn_ctx *ctx, bool force)
 {
   if (ctx->impl) {
     ctx->impl->force_match_escalation = force;
@@ -1772,7 +1771,7 @@ get_n_workers(grn_ctx *ctx, const char *p, const char *pe)
 #define HTTP_QUERY_PAIR_DELIMITER   "="
 #define HTTP_QUERY_PAIRS_DELIMITERS "&;"
 
-static grn_inline int
+static inline int
 command_proc_p(grn_obj *expr)
 {
   return (expr->header.type == GRN_PROC &&
@@ -2067,7 +2066,7 @@ grn_ctx_sendv(grn_ctx *ctx, int argc, char **argv, int flags)
   GRN_API_RETURN(ctx->rc);
 }
 
-static int
+static bool
 comment_command_p(const char *command, unsigned int length)
 {
   const char *p, *e;
@@ -2076,15 +2075,15 @@ comment_command_p(const char *command, unsigned int length)
   for (p = command; p < e; p++) {
     switch (*p) {
     case '#':
-      return GRN_TRUE;
+      return true;
     case ' ':
     case '\t':
       break;
     default:
-      return GRN_FALSE;
+      return false;
     }
   }
-  return GRN_FALSE;
+  return false;
 }
 
 unsigned int
