@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -85,16 +86,6 @@
 #  endif
 #endif
 
-#ifdef WIN32
-#  if defined(__cplusplus) || defined(__GNUC__)
-#    define grn_inline inline
-#  else /* defined(__cplusplus) || defined(__GNUC__) */
-#    define grn_inline _inline
-#  endif /* defined(__cplusplus) || defined(__GNUC__) */
-#else    /* WIN32 */
-#  define grn_inline inline
-#endif /* WIN32 */
-
 #ifdef WIN32
 #  ifndef NOMINMAX
 #    define NOMINMAX
@@ -361,8 +352,8 @@ typedef int grn_cond;
         SERR("MUTEX_LOCK");                                                    \
       }                                                                        \
       grn_nanosleep(1000000);                                                  \
-    } while (GRN_TRUE);                                                        \
-  } while (GRN_FALSE)
+    } while (true);                                                            \
+  } while (false)
 
 /* format string for printf */
 #ifdef WIN32
@@ -428,6 +419,7 @@ typedef int grn_cond;
  */
 #  define GRN_ATOMIC_ADD_EX(p, i, r) \
   __asm__ __volatile__ ("lock xaddl %0, %1" : "=r"(r), "+m"(*p) : "0"(i))
+typedef uint32_t grn_bit_scan_rev_result;
 /*
  * GRN_BIT_SCAN_REV() finds the most significant 1 bit of `v'. Then, `r' is set
  * to the index of the found bit. Note that `v' must not be 0.
@@ -444,6 +436,7 @@ typedef int grn_cond;
 #  define GRN_ATOMIC_ADD_EX(p,i,r) \
   __asm__ __volatile__ ("\n1:\n\tlwarx %0, 0, %1\n\tadd %0, %0, %2\n\tstwcx. %0, 0, %1\n\tbne- 1b\n\tsub %0, %0, %2" : "=&r" (r) : "r" (p), "r" (i) : "cc", "memory")
 /* todo */
+typedef uint32_t grn_bit_scan_rev_result;
 #  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
 #  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # elif (defined(__sun) && defined(__SVR4)) /* ATOMIC ADD */
@@ -451,15 +444,18 @@ typedef int grn_cond;
 #  define GRN_ATOMIC_ADD_EX(p,i,r) \
   (r = atomic_add_32_nv(p, i) - i)
 /* todo */
+typedef uint32_t grn_bit_scan_rev_result;
 #  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
 #  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # elif defined(__ATOMIC_SEQ_CST) /* GCC atomic builtins */
 #  define GRN_ATOMIC_ADD_EX(p,i,r) \
   (r = __atomic_fetch_add(p, i, __ATOMIC_SEQ_CST))
+typedef uint32_t grn_bit_scan_rev_result;
 #  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
 #  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # else /* ATOMIC ADD */
 /* todo */
+typedef uint32_t grn_bit_scan_rev_result;
 #  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
 #  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # endif /* ATOMIC ADD */
@@ -510,6 +506,7 @@ typedef int grn_cond;
 /* clang-format on */
 
 /* todo */
+typedef unsigned long grn_bit_scan_rev_result;
 #  define GRN_BIT_SCAN_REV(v, r)                                               \
     do {                                                                       \
       if (!BitScanReverse(&r, v)) {                                            \
@@ -528,6 +525,7 @@ typedef int grn_cond;
 #    define GRN_SET_64BIT(p, v)        (void)atomic_swap_64(p, v)
 #  endif /* ATOMIC ADD */
 /* todo */
+typedef uint32_t grn_bit_scan_rev_result;
 #  define GRN_BIT_SCAN_REV(v, r)  for (r = 31; r && !((1 << r) & v); r--)
 #  define GRN_BIT_SCAN_REV0(v, r) GRN_BIT_SCAN_REV(v, r)
 
@@ -538,7 +536,7 @@ typedef uint8_t byte;
 #define GRN_ID_WIDTH 30
 
 #ifdef __GNUC__
-grn_inline static int
+static inline int
 grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 {
   for (;; ap++, bp++, as--, bs--) {
