@@ -1,19 +1,18 @@
-/*
-  Copyright (C) 2024  Sutou Kouhei <kou@clear-code.com>
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+// Copyright (C) 2024-2025  Sutou Kouhei <kou@clear-code.com>
+//
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+//
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
+//
+// You should have received a copy of the GNU Lesser General Public
+// License along with this library; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 
 #include "grn_language_model.hpp"
 #include "grn_db.h"
@@ -21,6 +20,7 @@
 #include "groonga/smart_obj.hpp"
 
 #ifdef GRN_WITH_LLAMA_CPP
+#  include <ggml-backend.h>
 #  include <llama.h>
 #endif
 
@@ -91,11 +91,15 @@ namespace grn {
     } // namespace
 #endif
 
+    static char ggml_backends_dir[GRN_ENV_BUFFER_SIZE];
     static char language_models_dir[GRN_ENV_BUFFER_SIZE];
 
     void
     init_from_env(void)
     {
+      grn_getenv("GRN_GGML_BACKENDS_DIR",
+                 ggml_backends_dir,
+                 GRN_ENV_BUFFER_SIZE);
       grn_getenv("GRN_LANGUAGE_MODELS_DIR",
                  language_models_dir,
                  GRN_ENV_BUFFER_SIZE);
@@ -136,21 +140,77 @@ namespace grn {
     };
 
     static ModelCache model_cache;
-#endif
 
-    void
-    init_external_libraries(void)
-    {
-#ifdef GRN_WITH_LLAMA_CPP
-      llama_log_set(log_callback, &grn_gctx);
-      llama_backend_init();
+    static bool initialized = false;
+    static std::once_flag initialize_once;
+
+    namespace {
+#  ifdef _WIN32
+      const char *
+      get_default_ggml_backends_dir(void)
+      {
+        static char *windows_ggml_backends_dir = nullptr;
+        static char windows_ggml_backends_dir_buffer[PATH_MAX];
+        if (!windows_ggml_backends_dir) {
+          auto base_dir = grn_windows_base_dir();
+          auto base_dir_length = strlen(base_dir);
+          grn_strcpy(windows_ggml_backends_dir_buffer, PATH_MAX, base_dir);
+          grn_strcat(windows_ggml_backends_dir_buffer, PATH_MAX, "/");
+          grn_strcat(windows_ggml_backends_dir_buffer,
+                     PATH_MAX,
+                     GRN_RELATIVE_GGML_BACKENDS_DIR);
+          windows_ggml_backends_dir = windows_ggml_backends_dir_buffer;
+        }
+        return windows_ggml_backends_dir;
+      }
+#  else
+      const char *
+      get_default_ggml_backends_dir(void)
+      {
+        return GRN_GGML_BACKENDS_DIR;
+      }
+#  endif
+
+      const char *
+      get_ggml_backends_dir(void)
+      {
+        if (ggml_backends_dir[0]) {
+          return ggml_backends_dir;
+        } else {
+          return get_default_ggml_backends_dir();
+        }
+      }
+
+      void
+      init_external_libraries(void)
+      {
+        llama_log_set(log_callback, &grn_gctx);
+        ggml_backend_load_all_from_path(get_ggml_backends_dir());
+        llama_backend_init();
+        initialized = true;
+      }
+
+      void
+      ensure_init_external_libraries(void)
+      {
+        std::call_once(initialize_once, init_external_libraries);
+      }
+    } // namespace
+#else
+    namespace {
+      void
+      ensure_init_external_libraries(void)
+      {
+      }
+    } // namespace
 #endif
-    }
 
     void
     fin_external_libraries(void)
     {
 #ifdef GRN_WITH_LLAMA_CPP
+      if (!initialized) return;
+
       model_cache.clear();
       llama_backend_free();
       llama_log_set(nullptr, nullptr);
@@ -161,27 +221,33 @@ namespace grn {
 #  ifdef _WIN32
     static char *windows_language_models_dir = NULL;
     static char windows_language_models_dir_buffer[PATH_MAX];
-    static const char *
-    default_system_language_models_dir(void)
-    {
-      if (!windows_language_models_dir) {
-        const char *base_dir;
-        const char *relative_path = GRN_RELATIVE_LANGUAGE_MODELS_DIR;
+    namespace {
+      const char *
+      default_system_language_models_dir(void)
+      {
+        if (!windows_language_models_dir) {
+          const char *base_dir;
+          const char *relative_path = GRN_RELATIVE_LANGUAGE_MODELS_DIR;
 
-        base_dir = grn_windows_base_dir();
-        grn_strcpy(windows_language_models_dir_buffer, PATH_MAX, base_dir);
-        grn_strcat(windows_language_models_dir_buffer, PATH_MAX, "/");
-        grn_strcat(windows_language_models_dir_buffer, PATH_MAX, relative_path);
-        windows_language_models_dir = windows_language_models_dir_buffer;
+          base_dir = grn_windows_base_dir();
+          grn_strcpy(windows_language_models_dir_buffer, PATH_MAX, base_dir);
+          grn_strcat(windows_language_models_dir_buffer, PATH_MAX, "/");
+          grn_strcat(windows_language_models_dir_buffer,
+                     PATH_MAX,
+                     relative_path);
+          windows_language_models_dir = windows_language_models_dir_buffer;
+        }
+        return windows_language_models_dir;
       }
-      return windows_language_models_dir;
-    }
+    } // namespace
 #  else
-    static const char *
-    default_system_language_models_dir(void)
-    {
-      return GRN_LANGUAGE_MODELS_DIR;
-    }
+    namespace {
+      const char *
+      default_system_language_models_dir(void)
+      {
+        return GRN_LANGUAGE_MODELS_DIR;
+      }
+    } // namespace
 #  endif
 
     const char *
@@ -215,7 +281,7 @@ namespace grn {
   public:
     Impl(llama_model *model) : model_(model) {}
 
-    ~Impl(void) { llama_free_model(model_); }
+    ~Impl(void) { llama_model_free(model_); }
 
     llama_model *
     get_raw()
@@ -257,7 +323,7 @@ namespace grn {
         llama_model *raw_model;
         {
           language_model::CaptureError capture(ctx_);
-          raw_model = llama_load_model_from_file(model_path.c_str(), params);
+          raw_model = llama_model_load_from_file(model_path.c_str(), params);
           if (!raw_model) {
             GRN_LM_ERROR(ctx_,
                          GRN_INVALID_ARGUMENT,
@@ -271,7 +337,7 @@ namespace grn {
                 "[language-model-loader][load] encoder-decoder model isn't "
                 "supported yet: <%s>",
                 model_path.c_str());
-            llama_free_model(raw_model);
+            llama_model_free(raw_model);
             return nullptr;
           }
         }
@@ -310,7 +376,7 @@ namespace grn {
         model_(std::move(model)),
         llama_ctx_(llama_ctx),
         llama_model_(llama_get_model(llama_ctx_)),
-        n_dimentions_(llama_n_embd(llama_model_)),
+        n_dimentions_(llama_model_n_embd(llama_model_)),
         has_encoder_(llama_model_has_encoder(llama_model_)),
         has_decoder_(llama_model_has_decoder(llama_model_)),
         pooling_type_(llama_pooling_type(llama_ctx_))
@@ -453,7 +519,8 @@ namespace grn {
       if (tokens.capacity() < static_cast<size_t>(n_tokens)) {
         tokens.reserve(n_tokens);
       }
-      n_tokens = llama_tokenize(model,
+      auto vocab = llama_model_get_vocab(model);
+      n_tokens = llama_tokenize(vocab,
                                 text.data(),
                                 text.length(),
                                 tokens.data(),
@@ -463,7 +530,7 @@ namespace grn {
       if (n_tokens < 0) {
         // If guessed size isn't enough, use the real size.
         tokens.resize(-n_tokens);
-        llama_tokenize(model,
+        llama_tokenize(vocab,
                        text.data(),
                        text.length(),
                        tokens.data(),
@@ -588,6 +655,7 @@ namespace grn {
   {
 #ifdef GRN_WITH_LLAMA_CPP
     auto params = llama_context_default_params();
+    params.n_ctx = 0; // Use model's value
     params.embeddings = true;
     // We want document vector not token vectors. We want to use the
     // default pooling type in a model but it seems that most models
@@ -599,7 +667,7 @@ namespace grn {
       new LanguageModelInferencer::Impl(
         ctx,
         shared_from_this(),
-        llama_new_context_with_model(impl_->get_raw(), params)));
+        llama_init_from_model(impl_->get_raw(), params)));
 #else
     ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
         "[language-model][make-inferencer] llama.cpp isn't enabled");
@@ -632,6 +700,7 @@ struct grn_language_model_loader_ {
 grn_language_model_loader *
 grn_language_model_loader_open(grn_ctx *ctx)
 {
+  grn::language_model::ensure_init_external_libraries();
   auto loader = new grn_language_model_loader_(ctx);
   return loader;
 }
@@ -704,6 +773,8 @@ grn_language_model_close(grn_ctx *ctx, grn_language_model *model)
 grn_language_model_inferencer *
 grn_language_model_open_inferencer(grn_ctx *ctx, grn_language_model *model)
 {
+  grn::language_model::ensure_init_external_libraries();
+
   GRN_API_ENTER;
   if (!model) {
     ERR(GRN_INVALID_ARGUMENT,
