@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -109,7 +110,7 @@ struct sort_key_ {
   grn_expr_executor expr_executor;
 };
 
-grn_inline static size_t
+inline static size_t
 grn_sort_key_size(sort_key_offset offset)
 {
   if (offset == KEY_BULK) {
@@ -119,7 +120,7 @@ grn_sort_key_size(sort_key_offset offset)
   }
 }
 
-grn_inline static uint32_t
+inline static uint32_t
 grn_sort_key_get_size(const uint8_t *size_raw,
                       sort_key_offset key_offset)
 {
@@ -130,7 +131,7 @@ grn_sort_key_get_size(const uint8_t *size_raw,
   }
 }
 
-grn_inline static void
+inline static void
 grn_sort_key_set_size(uint8_t *size_raw,
                       sort_key_offset key_offset,
                       uint32_t size)
@@ -207,7 +208,7 @@ struct sort_compare_data_ {
   }\
 } while (0)
 
-static grn_inline grn_id
+static inline grn_id
 sort_value_get_resolve_id(grn_ctx *ctx,
                           const sort_key *key,
                           sort_entry *entry,
@@ -336,7 +337,7 @@ sort_value_execute_expr(grn_ctx *ctx,
   }
 }
 
-static grn_inline int
+static inline int
 sort_value_compare(grn_ctx *ctx,
                    sort_entry *a,
                    sort_entry *b,
@@ -424,7 +425,7 @@ sort_value_compare(grn_ctx *ctx,
   return 0;
 }
 
-static grn_inline void
+static inline void
 sort_value_swap(grn_ctx *ctx, sort_entry *a, sort_entry *b, sort_data *data)
 {
   sort_entry c = *a;
@@ -432,7 +433,7 @@ sort_value_swap(grn_ctx *ctx, sort_entry *a, sort_entry *b, sort_data *data)
   *b = c;
 }
 
-static grn_inline sort_entry *
+static inline sort_entry *
 sort_value_part(grn_ctx *ctx,
                 sort_entry *b,
                 sort_entry *e,
@@ -470,7 +471,7 @@ sort_value_part(grn_ctx *ctx,
   return e;
 }
 
-static grn_inline void
+static inline void
 sort_value_body(grn_ctx *ctx,
                 sort_entry *head,
                 sort_entry *tail,
@@ -490,7 +491,7 @@ sort_value_body(grn_ctx *ctx,
   }
 }
 
-static grn_inline sort_entry *
+static inline sort_entry *
 sort_value_pack(grn_ctx *ctx,
                 grn_obj *table,
                 sort_entry *head,
@@ -1147,7 +1148,7 @@ grn_table_sort_key_from_str_geo(grn_ctx *ctx, const char *str, unsigned int str_
             GRN_TEXT_INIT(&buf, GRN_OBJ_DO_SHALLOW_COPY);
             GRN_TEXT_SET(ctx, &buf, p + 1, r - p - 2); /* should be quoted */
             k->key = grn_obj_open(ctx, GRN_BULK, 0, domain);
-            grn_obj_cast(ctx, &buf, k->key, GRN_FALSE);
+            grn_obj_cast(ctx, &buf, k->key, false);
             GRN_OBJ_FIN(ctx, &buf);
           }
           k->flags |= GRN_TABLE_SORT_GEO;
