@@ -4,7 +4,8 @@
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
-# License version 2.1 as published by the Free Software Foundation.
+# License as published by the Free Software Foundation; either
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,9 +49,11 @@ function setup_with_apt () {
       package_names+=(
         ccache
         cmake
+        doxygen
         g++
         gcc
         gettext
+        graphviz
         libarrow-dev
         libedit-dev
         liblz4-dev
@@ -120,6 +123,8 @@ function setup_with_dnf () {
 
       package_names+=(
         ccache
+        doxygen
+        graphviz
         mecab-devel
         msgpack-devel
         simdjson-devel
