@@ -41,6 +41,9 @@ fi
 
 . "${build_top_dir}/config.sh"
 
+GRN_GGML_BACKENDS_DIR="$build_top_dir/bin"
+export GRN_GGML_BACKENDS_DIR
+
 GRN_PLUGINS_DIR="$source_top_dir/plugins"
 GRN_PLUGINS_PATH="$build_top_dir/plugins"
 export GRN_PLUGINS_DIR
@@ -148,6 +151,7 @@ have_targets="false"
 use_gdb="false"
 use_rr="false"
 use_valgrind="false"
+benchmark_mode="false"
 next_argument_is_long_option_value="false"
 for argument in "$@"; do
   case "$argument" in
@@ -168,6 +172,10 @@ for argument in "$@"; do
       # no argument options
       use_valgrind="true"
       ;;
+    --benchmark)
+      # no argument options
+      benchmark_mode="true"
+      ;;
     --*)
       next_argument_is_long_option_value="true"
       continue
@@ -195,7 +203,11 @@ if test "$CI" = "true"; then
 fi
 grntest_options+=("$@")
 if test "$have_targets" != "true"; then
-  grntest_options+=("${SOURCE_DIR}/suite")
+  if test "$benchmark_mode" = "true"; then
+    grntest_options+=("${SOURCE_DIR}/suite_benchmark")
+  else
+    grntest_options+=("${SOURCE_DIR}/suite")
+  fi
 fi
 
 tmpfs_candidates=("/dev/shm" "/run/shm")
