@@ -8,13 +8,15 @@ case "$0" in
     ;;
 esac
 
-if test -f version; then
-  GRN_VN=$(cat version)
+if test -f full_version; then
+  GRN_VN=$(cat full_version)
 elif test -d .git -o -f .git; then
   GRN_VN=$(git describe --abbrev=7 HEAD 2>/dev/null)
   if [ $? -ne 0 ]; then
     GRN_VN=$(cat base_version)
   fi
+else
+  GRN_VN=$(cat base_version)
 fi
 
 GRN_VN=$(expr "$GRN_VN" : v*'\(.*\)')
